package dev.kske.chess.ui;

import java.awt.BorderLayout;
import java.awt.GridLayout;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;

import dev.kske.chess.board.Piece.Color;
import dev.kske.chess.game.Game;
import dev.kske.chess.game.NaturalPlayer;

/**
 * Project: <strong>Chess</strong><br>
 * File: <strong>GamePane.java</strong><br>
 * Created: <strong>23.08.2019</strong><br>
 * Author: <strong>Kai S. K. Engelbart</strong>
 */
public class GamePane extends JComponent {

	private static final long serialVersionUID = 4349772338239617477L;

	private JButton		btnRestart, btnSwapColors;
	private BoardPane	boardPane;
	private Game		game;
	private Color		activeColor;

	public GamePane() {
		setLayout(new BorderLayout());
		boardPane = new BoardPane();
		add(boardPane, BorderLayout.CENTER);

		JPanel toolPanel = new JPanel();
		btnRestart = new JButton("Restart");
		btnRestart.addActionListener((evt) -> { if (game != null) game.reset(); game.start(); });

		activeColor		= Color.WHITE;
		btnSwapColors	= new JButton("Play as black");
		btnSwapColors.addActionListener((evt) -> {
			game.swapColors();
			btnSwapColors.setText("Play as " + activeColor.toString().toLowerCase());
			activeColor = activeColor.opposite();
		});

		toolPanel.add(btnRestart);
		toolPanel.add(btnSwapColors);
		add(toolPanel, BorderLayout.NORTH);

		JPanel letterPanel = new JPanel(new GridLayout(1, 8));
		for (int i = 0; i < 8; i++) {
			JLabel letterLabel = new JLabel(String.valueOf((char) (65 + i)));
			letterLabel.setHorizontalAlignment(JLabel.CENTER);
			letterPanel.add(letterLabel);
		}
		add(letterPanel, BorderLayout.SOUTH);

		JPanel numberPanel = new JPanel(new GridLayout(8, 1));
		for (int i = 0; i < 8; i++)
			numberPanel.add(new JLabel(String.valueOf(8 - i)));
		add(numberPanel, BorderLayout.EAST);
	}

	public BoardPane getBoardPane() { return boardPane; }

	/**
	 * @return The {@link Game} instance associated with this board pane
	 */
	public Game getGame() { return game; }

	/**
	 * Assigns a new {@link Game} instance to this game pane. If exactly one of the
	 * players is natural, color swapping functionality is enabled.
	 * 
	 * @param game The {@link Game} to assign to this game pane.
	 */
	public void setGame(Game game) {
		if (this.game != null) this.game.disconnect();
		this.game = game;
		btnSwapColors.setEnabled(game.getPlayers().get(Color.WHITE) instanceof NaturalPlayer
				^ game.getPlayers().get(Color.BLACK) instanceof NaturalPlayer);
	}
}
