package dev.kske.chess.ui;

import java.awt.EventQueue;
import java.awt.Toolkit;
import java.awt.dnd.DropTarget;

import javax.swing.JFrame;
import javax.swing.JTabbedPane;

/**
 * Project: <strong>Chess</strong><br>
 * File: <strong>MainWindow.java</strong><br>
 * Created: <strong>01.07.2019</strong><br>
 * Author: <strong>Kai S. K. Engelbart</strong>
 */
public class MainWindow {

	private JFrame		frame;
	private JTabbedPane	tabbedPane;

	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		EventQueue.invokeLater(() -> {
			try {
				new MainWindow();
			} catch (Exception ex) {
				ex.printStackTrace();
			}
		});
	}

	/**
	 * Create the application.
	 */
	public MainWindow() {
		initialize();
	}

	/**
	 * Initialize the contents of the frame.
	 */
	private void initialize() {
		// Configure frame
		frame = new JFrame("Chess by Kai S. K. Engelbart");
		frame.setResizable(false);
		frame.setBounds(100, 100, 494, 565);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setIconImage(Toolkit.getDefaultToolkit().getImage(getClass().getResource("/pieces/queen_white.png")));

		// Add frame content
		tabbedPane = new JTabbedPane();
		tabbedPane.add("Game 1", new GamePane());
		frame.getContentPane().add(tabbedPane);

		frame.setJMenuBar(new MenuBar(this));
		new DropTarget(frame, new FENDropTarget(this));

		// Update position and dimensions
		frame.pack();
		frame.setLocationRelativeTo(null);
		frame.setVisible(true);
	}

	/**
	 * @return The currently selected {@link GamePane} component
	 */
	public GamePane getSelectedGamePane() { return (GamePane) tabbedPane.getSelectedComponent(); }
}
